/* Musical Blob
 * University of Washington Computer Science and Engineering
 * Sound Capstone (CSE 481i)
 * Project Members: Nathan Armstrong, Daniel Chesney, Andrew Foster
 *
 * HandTracker
 * For documentation, see HandTracker.hpp
 */

#include <algorithm>
#include "HandTracker.hpp"

using std::min;
using std::max;

void HandTracker::listen(){
	osl.listen();
}

bool HandTracker::isTrackingHands() const{
	OscSkeletonListener::State s = osl.getState();
	return s == OscSkeletonListener::DATA_OLD
	    || s == OscSkeletonListener::DATA_NEW;
}

bool HandTracker::getHandData(TripleD& left, TripleD& right){
	if(state != CALIB_B) return false;
	try{
		left = mapper.mapPoint(lastLeft);
		right = mapper.mapPoint(lastRight);
	}catch(ZeroDomain zd){
		resetCalibration();
		return false;
	}
	return true;
}

void HandTracker::update(){
	if(osl.getState() == OscSkeletonListener::DATA_NEW &&
	   state == CALIB_B){
		TripleD torso;
		osl.getData(lastLeft, lastRight, torso);
		//lastLeft = lastLeft - torso;
		//lastRight = lastRight - torso;
	}
}

void HandTracker::forceUpdate(){
	TripleD torso;
	osl.getData(lastLeft, lastRight, torso);
	//lastLeft = lastLeft - torso;
	//lastRight = lastRight - torso;
}

static TripleD minEach(const TripleD& a, const TripleD& b){
	return TripleD(min(a.getElemA(), b.getElemA()),
	               min(a.getElemB(), b.getElemB()),
	               min(a.getElemC(), b.getElemC()));
}

static TripleD maxEach(const TripleD& a, const TripleD& b){
	return TripleD(max(a.getElemA(), b.getElemA()),
	               max(a.getElemB(), b.getElemB()),
	               max(a.getElemC(), b.getElemC()));
}

void HandTracker::triggerCalibration(const TripleD& minRange,
                                     const TripleD& maxRange){
	if(!isTrackingHands()) return;
	static TripleD minPoint, maxPoint;
	forceUpdate();
	if(state == NO_CALIB){
		minPoint = minEach(lastLeft, lastRight);
		maxPoint = maxEach(lastLeft, lastRight);
		state = CALIB_A;
	}else if(state == CALIB_A || state == CALIB_B){
		minPoint = minEach(minEach(lastLeft, lastRight), minPoint);
		maxPoint = maxEach(maxEach(lastLeft, lastRight), maxPoint);
		state = CALIB_B;
	}
	if(state == CALIB_B){
		TripleD rangeMargin = (maxRange - minRange).scale(0.25);
		mapper.mapRefA(minPoint, minRange - rangeMargin);
		mapper.mapRefB(maxPoint, maxRange + rangeMargin);
	}
}

void HandTracker::resetCalibration(){
	state = NO_CALIB;
}
