/* Musical Blob
 * University of Washington Computer Science and Engineering
 * Sound Capstone (CSE 481i)
 * Project Members: Nathan Armstrong, Daniel Chesney, Andrew Foster
 *
 * KinectInputHandler
 * Receives and handles OSCeleton messages on a specific port, and uses the
 * data to track the limb positions of users.
 */

//TODO: Correct concurrency errors (readers/writer pattern or flat
//monitor mutex)

#ifndef __KINECT_INPUT_HANDLER_HPP__
#define __KINECT_INPUT_HANDLER_HPP__

#include <omp.h>
#include "TripleD.hpp"
#include "LimbTracker.hpp"
#include "osc/OscPacketListener.h"
#include "ip/UdpSocket.h"

class KinectInputHandler : public osc::OscPacketListener{
public:
	//Creates a new KinectInputHandler that will listen for OSCeleton messages
	//on the given port, and using the given LimbTracker to store limb data.
	//LimbTracker is not taken possession of.
	KinectInputHandler(unsigned short listenPort, LimbTracker* track);
	~KinectInputHandler();

	//Runs a blocking listening loop on the given socket.  This does not return
	void listen(){ sock.Run(); }

private:
	LimbTracker* lt;
	UdpListeningReceiveSocket sock;
	omp_lock_t lock;

	//An automated callback for any OSCeleton messages received.
	virtual void ProcessMessage(const osc::ReceivedMessage& m,
	                            const IpEndpointName& remoteEndpoint);

	//Event handlers for specific message types.
	void jointEvent(const osc::ReceivedMessage& m);
	void loseUserEvent(const osc::ReceivedMessage& m);
};

#endif
