/* Musical Blob
 * University of Washington Computer Science and Engineering
 * Sound Capstone (CSE 481i)
 * Project Members: Nathan Armstrong, Daniel Chesney, Andrew Foster
 *
 * LimbTracker
 * Keeps track of position and velocity data for sets of limbs attached to
 * specific user IDs.
 */

#ifndef __LIMB_TRACKER_HPP__
#define __LIMB_TRACKER_HPP__

#include <map>
#include <set>
#include "TripleD.hpp"

using std::map;
using std::set;

//Types of limbs handled by the tracker
enum LimbType{
	RIGHT_HAND = 0,
	LEFT_HAND = 1,
	NUM_LIMBS = 2
};

class LimbTracker{
public:
	//Returns whether any limbs with the given user ID exists
	bool limbSetExists(unsigned id) const;
	//Returns whether the user with the given ID is tracking the given limb
	bool limbDataExists(unsigned id, LimbType limb) const;
	//Assigns the position of the given user's limb to posOut iff
	//limbDataExists returns true.  Returns whether the position was assigned.
	bool getLimbPos(unsigned id, LimbType limb, TripleD& posOut) const;
	//Assigns the velocity of the given user's limb to velOut iff
	//limbDataExists returns true.  Returns whether the velocity was assigned.
	bool getLimbVel(unsigned id, LimbType limb, TripleD& velOut) const;

	//Returns the set of user IDs currently being tracked
	const set<unsigned>& getUserIDs() const { return setIDs; }

	//If the given limb is already being tracked, updates its position.
	//Otherwise, begins tracking the limb at the given position.
	void updateLimbPos(unsigned id, LimbType limb, TripleD pos);
	//Stops tracking all limbs with the given user ID.
	void removeLimbSet(unsigned id);

private:
	class Limb;
	class LimbSet;

	set<unsigned> setIDs;
	map<unsigned, LimbSet*> sets;
};

//An exception class, used when an illegal limb type is passed to a method.
//Currently only used internally.
class IllegalLimbType {};

#endif
