/* Musical Blob
 * University of Washington Computer Science and Engineering
 * Sound Capstone (CSE 481i)
 * Project Members: Nathan Armstrong, Daniel Chesney, Andrew Foster
 *
 * OscSkeletonListener
 * For documentation, see OscSkeletonListener.hpp
 */

#include <cstring>
#include <iostream>
#include <cstdio>
#include "OscSkeletonListener.hpp"

void OscSkeletonListener::getData(TripleD& leftOut, TripleD& rightOut,
                                  TripleD& torsoOut){
	leftOut = leftPos;
	rightOut = rightPos;
	torsoOut = torsoPos;
	state = DATA_OLD;
}

void OscSkeletonListener::ProcessMessage(
		const osc::ReceivedMessage& m,
		const IpEndpointName& remoteEndpoint){
	try{
		const char* addr = m.AddressPattern();
		if(!strcmp(addr, "/new_user"))
			newUserEvent(m);
		else if(!strcmp(addr, "/new_skel"))
			newSkelEvent(m);
		else if(!strcmp(addr, "/joint"))
			jointEvent(m);
		else if(!strcmp(addr, "/lost_user"))
			loseUserEvent(m);
	}catch(osc::Exception& e){
		std::cerr << "Error while parsing message: " <<
		             m.AddressPattern() << ": " << e.what() << "\n";
	}
}

void OscSkeletonListener::newUserEvent(const osc::ReceivedMessage& m){
	if(state == PEND_USER){
		osc::int32 temp;
		m.ArgumentStream() >> temp >> osc::EndMessage;
		userID = temp;
		state = PEND_CALIB;
	}
}

void OscSkeletonListener::newSkelEvent(const osc::ReceivedMessage& m){
	if(state == PEND_CALIB){
		osc::int32 temp;
		m.ArgumentStream() >> temp >> osc::EndMessage;
		if((unsigned)temp == userID) state = DATA_OLD;
	}
}

enum JointType{
	UNK_JOINT,
	LEFT_HAND,
	RIGHT_HAND,
	TORSO
};

static JointType getJointType(const char* joint){
	if(strcmp(joint, "l_hand") == 0) return LEFT_HAND;
	else if(strcmp(joint, "r_hand") == 0) return RIGHT_HAND;
	else if(strcmp(joint, "torso") == 0) return TORSO;
	else return UNK_JOINT;
}

void OscSkeletonListener::jointEvent(const osc::ReceivedMessage& m){
	if(state == DATA_OLD || state == DATA_NEW){
		osc::int32 temp;
		const char* joint;
		float x, y, z;
		osc::ReceivedMessageArgumentStream args = m.ArgumentStream();

		args >> joint;
		char jointID = getJointType(joint);
		if(jointID != UNK_JOINT){
			args >> temp;
			if((unsigned)temp == userID){
				args >> x >> y >> z >> osc::EndMessage;
				TripleD pos(x, y, z);
				if(jointID == LEFT_HAND) leftPos = pos;
				else if(jointID == RIGHT_HAND) rightPos = pos;
				else if(jointID == TORSO) torsoPos = pos;
			}
			state = DATA_NEW;
		}
	}
}

void OscSkeletonListener::loseUserEvent(const osc::ReceivedMessage& m){
	if(state != PEND_USER){
		osc::int32 temp;
		m.ArgumentStream() >> temp >> osc::EndMessage;
		if((unsigned)temp == userID)
			state = PEND_USER;
	}
}

