/* Musical Blob
 * University of Washington Computer Science and Engineering
 * Sound Capstone (CSE 481i)
 * Project Members: Nathan Armstrong, Daniel Chesney, Andrew Foster
 *
 * Particle
 * This represents a generic particle, holding current and previous position,
 * as well as a force accumulator for aid in physics calculations.
 */

#ifndef __PARTICLE_H__
#define __PARTICLE_H__

#include "TripleD.hpp"

class Particle{
public:
	//Create a particle at the given position
	Particle(TripleD initPos)
		: currPos(initPos), prevPos(initPos), forceAccum(), physActive(true) {}
	Particle(double initPosX, double initPosY, double initPosZ)
	  : currPos(initPosX, initPosY, initPosZ), prevPos(currPos), forceAccum(),
	    physActive(true) {}

	//Accessor functions
	inline void setCurrPos(TripleD t){ currPos = t; }
	inline void setPrevPos(TripleD t){ prevPos = t; }
	inline void setPhysicsActive(bool b){ physActive = b; }

	inline const TripleD& getCurrPos() const { return currPos; }
	inline const TripleD& getPrevPos() const { return prevPos; }
	inline const TripleD& getForceAccum() const { return forceAccum; }

	//Returns whether forces will be applied when updating this particle
	inline const bool isPhysicsActive() const { return physActive; }

	//Clears the force accumulator
	void clearForceAccum();

	//Adds the given triple to the force accumulator
	void addForce(TripleD f);

	//Updates particle state using the given timestep
	void updateParticle(double tssquare);

protected:
	TripleD currPos, prevPos, forceAccum;
	bool physActive;
};

#endif
