/* Musical Blob
 * University of Washington Computer Science and Engineering
 * Sound Capstone (CSE 481i)
 * Project Members: Nathan Armstrong, Daniel Chesney, Andrew Foster
 *
 * ParticleBoundSet
 * Keeps track of a set of particles, with standard union operator.
 * Can determine whether its own bounding box intersects with another set's,
 * which can be leveraged for merging sets that in proximity to each other.
 */

#ifndef __PARTICLE_BOUND_SET_HPP__
#define __PARTICLE_BOUND_SET_HPP__

#include <vector>
#include "Particle.hpp"
#include "BoundingBox.hpp"

using std::vector;

class ParticleBoundSet{
public:
	//Creates a set containing only the given particle, with the given margin
	//around the particle on all sides as a bounding box.
	//Requires the particle is not null.
	ParticleBoundSet(Particle* p, double margin);

	//Returns a vector containing the particles in this set
	const vector<Particle*>& getParts() const { return parts; }
	//Returns the minimum point of the bounding box for this set
	TripleD getMinBound() const { return bounds.getMinCorner(); }
	//Returns the maximum point of the bounding box for this set
	TripleD getMaxBound() const { return bounds.getMaxCorner(); }

	//Returns whether the bounding box of this set intersects that of the
	//given set
	bool intersects(const ParticleBoundSet* const pbs) const;

	//Merges the particles and bounding box of this set and the provided set
	void mergeSubset(const ParticleBoundSet* const pbs);

private:
	vector<Particle*> parts;
	BoundingBox bounds;
};

#endif
