/* Musical Blob
 * University of Washington Computer Science and Engineering
 * Sound Capstone (CSE 481i)
 * Project Members: Nathan Armstrong, Daniel Chesney, Andrew Foster
 *
 * PointMapper
 * The mapper takes two sample points each in the domain and range, and uses
 * them to create a linear mapping from any point in the domain to its
 * counterpart in the range.
 */

#ifndef __POINT_MAPPER_HPP__
#define __POINT_MAPPER_HPP__

#include "TripleD.hpp"

class PointMapper{
public:
	//Creates a new PointMapper.  If clip is true, mapped points are restricted
	//to fall within the area defined by the two provided range points.
	//Otherwise, points will simply be extrapolated outside.
	//If left undeclared, no clipping occurs.
	PointMapper(bool clip = false) : clipToRange(clip) {}

	//Sets sample domain/range pair A and B, respectively.
	void mapRefA(const TripleD dom, const TripleD rng);
	void mapRefB(const TripleD dom, const TripleD rng);

	//If either domain sample point shares the same value along any axis, the
	//following functions throw a ZeroDomain object.

	//Maps the given point from the domain to the range.
	TripleD mapPoint(const TripleD point) const;
	//Scales the given point without translating.  Effective for vector scaling
	TripleD scalePoint(const TripleD point) const;

private:
	TripleD domA, domB;
	TripleD rngA, rngB;

	TripleD convSlope, convInter;

	bool clipToRange;

	//Helper function for setting up the transformation function
	void formLines();
};

//Exception for when the provided domain region has zero size along any axis.
class ZeroDomain{};

#endif
