/* Musical Blob
 * University of Washington Computer Science and Engineering
 * Sound Capstone (CSE 481i)
 * Project Members: Nathan Armstrong, Daniel Chesney, Andrew Foster
 *
 * TripleD
 * For documentation, see TripleD.hpp
 */

#include <cmath>
#include "TripleD.hpp"

TripleD TripleD::operator+(const TripleD &rhs) const{
	return TripleD(elemA + rhs.elemA, elemB + rhs.elemB, elemC + rhs.elemC);
}

TripleD TripleD::operator-(const TripleD &rhs) const{
	return TripleD(elemA - rhs.elemA, elemB - rhs.elemB, elemC - rhs.elemC);
}

TripleD& TripleD::operator+=(const TripleD &rhs){
	elemA += rhs.elemA;
	elemB += rhs.elemB;
	elemC += rhs.elemC;
	return *this;
}

TripleD& TripleD::operator-=(const TripleD &rhs){
	elemA -= rhs.elemA;
	elemB -= rhs.elemB;
	elemC -= rhs.elemC;
	return *this;
}

TripleD& TripleD::operator=(const TripleD &rhs){
	if(&rhs != this){
		elemA = rhs.elemA;
		elemB = rhs.elemB;
		elemC = rhs.elemC;
	}
	return *this;
}

double TripleD::operator*(const TripleD &rhs) const {
	return (elemA * rhs.elemA) + (elemB * rhs.elemB) + (elemC * rhs.elemC);
}

bool TripleD::operator==(const TripleD &rhs) const{
	return (elemA == rhs.elemA)
	    && (elemB == rhs.elemB)
	    && (elemC == rhs.elemC);
}

bool TripleD::operator!=(const TripleD &rhs) const{
	return !((*this) == rhs);
}

TripleD TripleD::scale(double d) const {
	return TripleD(elemA * d, elemB * d, elemC * d);
}

TripleD TripleD::scale(double aScale, double bScale, double cScale) const {
	return TripleD(elemA * aScale, elemB * bScale, elemC * cScale);
}

TripleD TripleD::scale(const TripleD& scales) const{
	return scale(scales.elemA, scales.elemB, scales.elemC);
}

TripleD TripleD::inverse() const{
	return TripleD(1/elemA, 1/elemB, 1/elemC);
}

double TripleD::magnitude() const {
	return sqrt((elemA * elemA) + (elemB * elemB) + (elemC * elemC));
}

TripleD TripleD::normalize() const {
	double invMag = 1/magnitude();
	return scale(invMag);
}
